function DTI_processing_demo()
%% This script provides sample codes for calculating DTI paramters from diffuison wegihted images
%Deqiang Qiu, qiudeqiang@gmail.com

%% Step 1 load the data and visualize it, 
im =[];
bval=[];
bvec = [];
load('DTI_data.mat');
figure(1);
%visualize the gradient encoding vectors, use rotation tool for inspection, they only span half hemisphere since mirrored directions have the same diffusion weighting ,
numDirections = size(bvec,1);%DWI+B0
quiver3(zeros([numDirections,1]),zeros([numDirections,1]),zeros([numDirections,1]),bvec(:,1),bvec(:,2),bvec(:,3));

% use im4Dview to explore the DWI data
im4Dview(im);
%% Step 2, calculate gradient encoding matrix
% V'*D*V = TRACE(V'*D*V) = TRACE(V*V'*D) = sum((V*V').*D') = sum((V*V').*D)

BMatrix = zeros([numDirections, 7]);%log(S) = BMatrix*Deff

for i = 1:numDirections
    V = bvec(i,:);
    bMat = V'*V;
    BMatrix(i,1) = bMat(1,1);
    BMatrix(i,2) = bMat(1,2)*2;%element-wise product of sysmetric matrices
    BMatrix(i,3) = bMat(1,3)*2;
    BMatrix(i,4) = bMat(2,2);
    BMatrix(i,5) = bMat(2,3)*2;
    BMatrix(i,6) = bMat(3,3);
    BMatrix(i,1:6) = BMatrix(i,1:6)*(-bval(i));%scale by b value
    BMatrix(i,7) = 1;%constant term for logS0
end

%% Step 3 reshape image for vectorization
szImg = size(im);
numVoxels = prod(szImg(1:3));
img = reshape(permute(im,[4,1,2,3]),[szImg(4),numVoxels]);

%calcuate Deff and logS0 by solving linear equation
Deff = BMatrix\log(img);%the first 6 columns are Deff and the last logS0

S0 = reshape(exp(Deff(7,:)),szImg(1:3));

%allocate memory for eigen directions (V1,2,3) and eigen diffusivities
%(D1,2,3)
V1 = zeros([3,numVoxels]);
V2 = V1;
V3 = V1;
D1 = zeros([1,numVoxels]);
D2 = D1;
D3 = D1;

%Dtmp=zeros(3,3);%3x3 tensor

for i = 1:numVoxels
    %skip voxels whose intensitis are really small
    if(img(1,i)<20) %100 is arbitrary
        continue;
    end
    Dtmp = [ Deff(1,i),Deff(2,i),Deff(3,i);...
             Deff(2,i),Deff(4,i),Deff(5,i);...
             Deff(3,i),Deff(5,i),Deff(6,i)];
    [U,S,V] = svd(Dtmp);%we could use eig as well since Dtmp is symmetric, but svd is more numerically stable
    S(S<0)=-S(S<0); %Dtmp should be positive definite, but because of noise and we did not enforced it during calculation, this may not hold

    V1(:,i) = U(:,1);
    V2(:,i) =U(:,2);
    V3(:,i) = U(:,3);
    D1(i) = S(1,1);
    D2(i) =S(2,2);
    D3(i) = S(3,3);
end

%% reshape to image dimensions
V1 = reshape(permute(V1,[2,1]),[szImg(1:3),3]);
V2 = reshape(permute(V2,[2,1]),[szImg(1:3),3]);
V3 = reshape(permute(V3,[2,1]),[szImg(1:3),3]);

D1 = reshape(D1,szImg(1:3))*1e6;%scale up
D2 = reshape(D2,szImg(1:3))*1e6;
D3 = reshape(D3,szImg(1:3))*1e6;

%% mean diffusivity, FA, etc
%S0 has been created already
MD = (D1+D2+D3)/3;
FA = sqrt(3/2)*(((D1-MD).^2+(D2-MD).^2+(D3-MD).^2)./(D1.^2 + D2.^2 + D3.^2)).^0.5;
%use im4DView to explore the images
im4Dview(FA);
%color FA
colorFA = abs(repmat(FA,[1,1,1,3]).*V1);
figure(2);
imshow(flipud(permute(squeeze(colorFA(:,:,40,:)),[2,1,3])));%im4Dview doesn't support RGB, use imshow here instead


end

