function h = im4Dview(im,rng,clbHandle)
%function h = im4Dview(im, rng, clbHandle)
%This function display 4D images, click on the image, the time series will be displayed
%im, 4D image matrix of nifti file name
%h, figure handle
%clbHandle, function for fitting function, fitting callback
%   prototype: [yFit] = clbHandle(pos, data)
%               where pos, is a three component vector specifying the current cursor position 
%                     data, the actual data vector along the 4th dimension                      
%
%Deqiang Qiu. qiudeqiang@gmail.com
%Stanford University. 2011
%



if(ischar(im))
    im = spm_read_vols(spm_vol(im));
end
im = permute(im,[2,1,3,4]);
sz = size(im);
if(numel(sz)<4)
    sz(numel(sz)+1:4) = 1;
end
if(~exist('rng','var')||isempty(rng))
    rng = [min(im(:)),max(im(:))];
end

if(~exist('clbHandle','var'))
    clbHandle = [];
end

%set up the view
h = imtool(squeeze(im(:,:,1,1)),rng);
hImg = findobj(h,'type','image');
hAxes = get(hImg,'parent');
set(hAxes,'ydir','normal');
%set(h,'unit','normalized');

%set the slider
hTime = uicontrol(h,'style','slider','min',1,'max',sz(4),'value',1,'sliderstep',1./[sz(4),max([1,sz(4)/2])],'units','normalized','position',[0.985,0.5,0.01,0.5],'tag','sliderTime','callback',{@im4DViewCallback,'updateView'});
   htextSlice = uicontrol(h,'Style','text','String','Slice Num:1',...
           'units','normalized','position',[0.9,0.4,0.1,0.05]);
        htextTime = uicontrol(h,'Style','text','String','Time Num:1',...
           'units','normalized','position',[0.9,0.3,0.1,0.05]);
if(sz(4)==1)
    set(hTime,'visible','off');
end
hSlice = uicontrol(h,'style','slider','min',1,'max',sz(3),'value',1,'sliderstep',1./[sz(3),max([1,sz(3)/2])],'units','normalized','position',[0.97,0.5,0.01,0.5],'tag','sliderSlice','callback',{@im4DViewCallback,'updateView'});

if(sz(3)==1)
    set(hSlice,'visible','off');
end

if(sz(4)>1)
    %set(hImg,'ButtonDownFcn',{@im4DViewCallback,'ButtonDown'});
    set(h,'WindowButtonDownFcn',{@im4DViewCallback,'ButtonDown'});
else
    set(h,'WindowButtonDownFcn',[]);
end


%create guidata
mydata.hFigure = h;
mydata.hImg = hImg;
mydata.hTime = hTime;
mydata.hSlice = hSlice;
mydata.hAxes = hAxes;
mydata.im = im;
mydata.hPlot = [];
mydata.hPlotFigure = [];
mydata.hPlotAxes = [];
mydata.hCursor = [];
mydata.bButtonDown = 0;
mydata.clbHandle = clbHandle;
mydata.hDefaultMotionFcn = get(h,'WindowButtonMotionFcn');
mydata.htextSlice = htextSlice;
mydata.htextTime = htextTime;

guidata(h,mydata);


return

function im4DViewCallback(src, event, arg1)
%disp(arg1)
mydata = guidata(gcbo);
    if(strcmpi(arg1,'updateView'))
        updateview(gcbo);
    elseif(strcmpi(arg1,'ButtonDown'))
        %disp('hello')
        %plotTimeCurve(gcbo);
        ButtonDown(gcbo);
    elseif(strcmpi(arg1,'ButtonMotion'))
        ButtonMotion(gcbo);
        mydata.hDefaultMotionFcn(src,event);
    elseif(strcmpi(arg1,'ButtonUp'))
        ButtonUp(gcbo);
    end
return

function updateview(hcbo)

mydata = guidata(hcbo);

slice = round(get(mydata.hSlice,'value'));
tm = round(get(mydata.hTime,'value'));

set(mydata.htextSlice,'String',sprintf('Slice Num:%d',slice));
set(mydata.htextTime,'String',sprintf('Time Num:%d',tm));

hImg = mydata.hImg;

set(hImg,'CData',squeeze(mydata.im(:,:,slice,tm)));
return

function ButtonDown(hcbo)
mydata = guidata(hcbo);
%    if(hcbo~=mydata.hImg)
%        return
%    end
%check that we are in the image
hAxes = mydata.hAxes;
curPoint = get(hAxes,'CurrentPoint');
curP = round(curPoint(1,1:2));
xlim = get(hAxes,'xlim');
ylim = get(hAxes,'ylim');
if(prod(xlim-curP(1))>0||prod(ylim-curP(2))>0)
    return
end

%start capture
set(mydata.hFigure,'WindowButtonMotionFcn',{@im4DViewCallback,'ButtonMotion'});
set(mydata.hFigure,'WindowButtonUpFcn',{@im4DViewCallback,'ButtonUp'});
%get(mydata.hFigure,'WindowButtonMotionFcn')

plotTimeCurve(hcbo);
%lock in the function


return


function ButtonUp(hcbo)
mydata = guidata(hcbo);
set(mydata.hFigure,'WindowButtonMotionFcn',mydata.hDefaultMotionFcn);
%set(mydata.hFigure,'WindowButtonUpFcn',[]);
return

function ButtonMotion(hcbo)
%mydata = guidata(hcbo);
plotTimeCurve(hcbo);
return




function plotTimeCurve(hcbo)
    mydata = guidata(hcbo);
%    if(hcbo~=mydata.hImg)
%        return
%    end
%check that we are in the image
hAxes = mydata.hAxes;
curPoint = get(hAxes,'CurrentPoint');
curP = round(curPoint(1,1:2));
xlim = get(hAxes,'xlim');
ylim = get(hAxes,'ylim');
if(prod(xlim-curP(1))>0||prod(ylim-curP(2))>0)
    return
end
curP = round(curP);

slice = round(get(mydata.hSlice,'value'));

ts = squeeze(mydata.im(curP(2),curP(1),slice,:));
if(isempty(mydata.hPlotAxes)||~ishandle(mydata.hPlotAxes))
    hf = figure;
    hPlot = plot(ts,'+-');
    mydata.hPlot = hPlot;
    mydata.hPlotAxes = get(hPlot,'parent');
    legend(strvcat('Data Points','fit'));
    mydata.hPlotFigure = hf;
    guidata(hcbo,mydata);
end
    %set(mydata.hPlot,'YData',ts);
    if(~isempty(mydata.clbHandle))
        fit = mydata.clbHandle([curP(1),curP(2),slice],ts');
        %plot(mydata.hPlotAxes,1:numel(ts),ts','+-',1:numel(ts),fit,'ro-');
        plot(mydata.hPlotAxes,1:numel(ts),fit,'+-');
    else
        plot(mydata.hPlotAxes,ts','+-');
    end



%redraw cursor
curX = [curP(1)-3,curP(1);
            curP(1)+3,curP(1)];
    curY = [curP(2),curP(2)-3;
            curP(2),curP(2)+3];
    
if(isempty(mydata.hCursor)||~ishandle(mydata.hCursor(1)))
    hCursor = line(curX,curY,'color','red','parent',mydata.hAxes);
    mydata.hCursor = hCursor;
    guidata(hcbo,mydata);
end
    set(mydata.hCursor(1),'XData',curX(:,1),'YData',curY(:,1));
    set(mydata.hCursor(2),'XData',curX(:,2),'YData',curY(:,2));
    
%figure(mydata.hPlotFigure);
      

return




